/*
    Scrollable Window for CXL.

    Copyright (c) 1990 Tony Tortorelli
*/
/*-----------------------------------------------------------------------------

    NAME
        wscrollwin

    DESCRIPTION
        This function will open up a window and display text lines
        in a window.  If all of the lines cannot fit in the window,
        scrolling and paging is allowed by use of the Uparrow,
        Downarrow, PgUp, PgDn,  Home and End keys.


    SYNOPSIS
        #include "cxlwin.h"
        int wscrollwin(int srow,int scol,int erow,int ecol,int btype,
                       int bordattr,int winattr,char **page)

    INPUTS
        srow       - start row of window
        scol       - start column of window
        erow       - end row of window
        ecol       - end column of window.
        btype      - window box type (0-5)
        bordattr   - border attribute
        winattr    - window attribute
        page       - address of array of char pointers (strings) to
                     display. The last pointer in the array MUST be NULL!

    RETURN VALUE
        If an error occurred, then -1 will be returned and
        _winfo.errno will be set to one of the following:

            W_ESCPRESS - the [Esc] key was pressed
            W_INVCOORD - invalid window coordinates
            W_INVBTYPE - invalid box type
            W_STRLONG  - window not wide enough to hold largest string

    EXAMPLE
        char *page[] = {"Line 1", "Line 2", "Line 3", "Line 4", "Line 5",
                        "Line 6", "Line 7", "Line 8", "Line 9", "Line 10",
                        "Line 11","Line 12","Line 13","Line 14","Line 15",
                        "Line 16","Line 17","Line 18","Line 19","Line 20",
                        "Line 21","Line 22","Line 23","Line 24","Line 25",
                        "Line 26","Line 26","Line 28","Line 29","Line 30",
                        NULL};

        wscrollwin(5,10,10,50,0,LCYAN|_BLUE,CYAN|_BLUE,page);

------------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <cxlwin.h>
#include <cxldef.h>
#include <cxlkey.h>

static WINDOW w1, w2;

int wscrollwin(int srow,int scol,int erow,int ecol,int btype,
               int bordattr,int winattr,char **page)
{
    register char **p;
    register int i,line;
    int totallines, pagelength, width, largestline=0, cl=0, quit=0;
    char cmdline[] = "  PgUp PgDn Home End Quit";

    /* calculate total lines requested */

    p = page;
    for(p = page, totallines=0; *p != NULL; p++, totallines++)
        if(largestline < strlen(*p))
            largestline = strlen(*p);

    /* calculate pagelength and width */

    if(btype == 5){
        width      = ecol - scol + 1;
        pagelength = erow - srow;
    }
    else{
        width      = ecol - scol - 1;
        pagelength = erow - srow - 2;
    }

    /* Test that winow is wide enough */

    if(largestline > width){
        _winfo.errno=W_STRLONG;
        return -1;
    }

    /* Test if Command Line is necessary and if the */
    /*  window is wide enough to accomodate it      */

    if(((ecol - scol + 1) > strlen(cmdline)) && (totallines > (pagelength+1)))
        cl = 1;
	else
    	pagelength++;	/* erow increased, so increase pagelenght */

    /* open data window */

    if((w1 = wopen(srow,scol,cl?(erow-1):(erow),ecol,btype,bordattr,winattr)) == 0)
        return -1;

    /* open "keys" window */

    if(cl){
        if((w2 = wopen(erow,scol,erow,ecol,5,bordattr,winattr)) == 0)
            return -1;
        wcenters(0,winattr,cmdline);
        wactiv(w1);     /* activate output window */
    }

    /* print first page */

    for(line=0; line < pagelength && page[line] != NULL; line++)
        wprints(line,0,winattr,page[line]);

    /* request user input */

    while(!quit){
        switch(getxch()){
            case 0x4700:    /* Home */
                if((line - pagelength) > 0){
                    wclear();   /* print first page */
                    for(line=0; line < pagelength && page[line] != NULL; line++)
                        wprints(line,0,winattr,page[line]);
                }
                break;
            case 0x4800:    /* Uparrow */
                if(line > pagelength){
                    wscroll(1,D_DOWN);
                    wprints(0,0,winattr,page[(--line)-pagelength]);
                }
                break;
            case 0x5000:    /* Downarrow    */
                if(page[line] != NULL){
                    wscroll(1,D_UP);
                    wprints(pagelength-1,0,winattr,page[line++]);
                }
                break;
            case 0x4900:    /* Page Up */
                if(line > pagelength){
                    wclear();
                    if((line - (pagelength * 2)) < 0)
                        line = 0;
                    else
                        line = line - (pagelength * 2);
                    for(i=0; i < pagelength && page[line] != NULL; i++, line++)
                        wprints(i,0,winattr,page[line]);
                }
                break;
            case 0x5100:     /* Page Down    */
                if(page[line] != NULL){
                    if((line + pagelength) > totallines){
                        line = totallines - (totallines % pagelength);
                        line = line - (pagelength - (totallines % pagelength));
                    }
                    wclear();
                    for(i=0; i < pagelength && page[line] != NULL; i++, line++)
                        wprints(i,0,winattr,page[line]);
                }
                break;
            case 0x4f00:        /* End */
                if(page[line] != NULL){
                    line = totallines - (totallines % pagelength);
                    if((line - ((pagelength - (totallines % pagelength)))) < 0)
                        line = 0;
                    else
                        line = line - ((pagelength - (totallines % pagelength)));
                    wclear();
                    for(i=0; i < pagelength && page[line] != NULL; i++, line++)
                        wprints(i,0,winattr,page[line]);
                }
                break;
            case 0x1071:    /*  ESC or Quit   */
            case 0x011b:
                quit = 1;
                break;
        }
    }
    wactiv(w1);     /* actvate output window*/
    wclose();       /* Close Display Window */
    if(cl){
        wactiv(w2);     /* actvate "Menu" window*/
        wclose();       /* Close Keys "Menu"    */
    }

    return 1;       /* Return Normally      */
}
